"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getUpdateChoices = void 0;
const colorize_semver_diff_1 = __importDefault(require("@pnpm/colorize-semver-diff"));
const semver_diff_1 = __importDefault(require("@pnpm/semver-diff"));
const table_1 = require("@zkochan/table");
const isEmpty_1 = __importDefault(require("ramda/src/isEmpty"));
const unnest_1 = __importDefault(require("ramda/src/unnest"));
function getUpdateChoices(outdatedPkgsOfProjects) {
    const allOutdatedPkgs = mergeOutdatedPkgs(outdatedPkgsOfProjects);
    if ((0, isEmpty_1.default)(allOutdatedPkgs)) {
        return [];
    }
    const rowsGroupedByPkgs = Object.entries(allOutdatedPkgs)
        .sort(([pkgName1], [pkgName2]) => pkgName1.localeCompare(pkgName2))
        .map(([pkgName, outdatedPkgs]) => ({
        pkgName,
        rows: outdatedPkgsRows(Object.values(outdatedPkgs)),
    }));
    const renderedTable = alignColumns((0, unnest_1.default)(rowsGroupedByPkgs.map(({ rows }) => rows)));
    const choices = [];
    let i = 0;
    for (const { pkgName, rows } of rowsGroupedByPkgs) {
        choices.push({
            message: renderedTable
                .slice(i, i + rows.length)
                .join('\n    '),
            name: pkgName,
        });
        i += rows.length;
    }
    return choices;
}
exports.getUpdateChoices = getUpdateChoices;
function mergeOutdatedPkgs(outdatedPkgs) {
    const allOutdatedPkgs = {};
    for (const outdatedPkg of outdatedPkgs) {
        if (!allOutdatedPkgs[outdatedPkg.packageName]) {
            allOutdatedPkgs[outdatedPkg.packageName] = {};
        }
        const key = JSON.stringify([
            outdatedPkg.latestManifest?.version,
            outdatedPkg.current,
        ]);
        if (!allOutdatedPkgs[outdatedPkg.packageName][key]) {
            allOutdatedPkgs[outdatedPkg.packageName][key] = outdatedPkg;
            continue;
        }
        if (allOutdatedPkgs[outdatedPkg.packageName][key].belongsTo === 'dependencies')
            continue;
        if (outdatedPkg.belongsTo !== 'devDependencies') {
            allOutdatedPkgs[outdatedPkg.packageName][key].belongsTo = outdatedPkg.belongsTo;
        }
    }
    return allOutdatedPkgs;
}
function outdatedPkgsRows(outdatedPkgs) {
    return outdatedPkgs
        .map((outdatedPkg) => {
        const sdiff = (0, semver_diff_1.default)(outdatedPkg.wanted, outdatedPkg.latestManifest.version);
        const nextVersion = sdiff.change === null
            ? outdatedPkg.latestManifest.version
            : (0, colorize_semver_diff_1.default)(sdiff); // eslint-disable-line @typescript-eslint/no-explicit-any
        let label = outdatedPkg.packageName;
        switch (outdatedPkg.belongsTo) {
            case 'devDependencies': {
                label += ' (dev)';
                break;
            }
            case 'optionalDependencies': {
                label += ' (optional)';
                break;
            }
        }
        return [label, outdatedPkg.current, '❯', nextVersion];
    });
}
function alignColumns(rows) {
    return (0, table_1.table)(rows, {
        border: (0, table_1.getBorderCharacters)('void'),
        columnDefault: {
            paddingLeft: 0,
            paddingRight: 1,
        },
        columns: {
            1: { alignment: 'right' },
        },
        drawHorizontalLine: () => false,
    }).split('\n');
}
//# sourceMappingURL=getUpdateChoices.js.map